//+------------------------------------------------------------------+
//|                                                   MBookPanel.mqh |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#include "Node.mqh"
#include <Panel\Events\EventChartEndEdit.mqh>
//+------------------------------------------------------------------+
//|     .            |
//|    .                     |
//+------------------------------------------------------------------+
class CElChart : public CNode
  {
private:
   ENUM_OBJECT       m_obj_type;      //  ,    

   long              m_x;             // X-  
   long              m_y;             // Y-  
   long              m_height;        //     
   long              m_width;         //    

   string            m_text;          //     
   int               m_text_size;     //   
   color             m_text_color;    //    
   string            m_text_font;     //   
   bool              m_read_only;     //   
   ENUM_ALIGN_MODE   m_align;         //    OBJ_EDIT

   color             m_bground_color; //   
   color             m_border_color;  //   
   ENUM_BORDER_TYPE  m_border_type;   //   
   ENUM_BASE_CORNER  m_corner;        //    

protected:
   virtual void      OnXCoordChange(void);
   virtual void      OnYCoordChange(void);
   virtual void      OnWidthChange(void);
   virtual void      OnHeightChange(void);
public:
                     CElChart(ENUM_OBJECT obj_type);
   virtual void      Show();
   virtual void      Event(CEvent *event);
/* Set methods */
   void              Text(string text);
   void              TextSize(int size);
   void              TextFont(string font);
   void              TextColor(color clr);
   void              ReadOnly(bool read_only);
   void              Align(ENUM_ALIGN_MODE align);

   void              BackgroundColor(color clr);
   void              BorderColor(color clr);
   void              BorderType(ENUM_BORDER_TYPE type);

   void              XCoord(long x);
   void              YCoord(long y);
   void              Width(long width);
   void              Height(long height);
   void              Corner(ENUM_BASE_CORNER corner);

/* Get methods */
   string            Text(void);
   int               TextSize(void);
   color             TextColor(void);
   string            TextFont(void);
   bool              ReadOnly(void);
   ENUM_ALIGN_MODE   Align(void);

   color             BackgroundColor(void);
   color             BorderColor(void);
   ENUM_BORDER_TYPE  BorderType(void);

   long              XCoord(void);
   long              YCoord(void);
   long              Width(void);
   long              Height(void);
   ENUM_BASE_CORNER  Corner(void);

   ENUM_OBJECT       TypeElement(void);

  };
//+------------------------------------------------------------------+
//|  -                                         |
//+------------------------------------------------------------------+
CElChart::CElChart(ENUM_OBJECT obj_type)
  {
   m_obj_type=obj_type;
   XCoord(20);
   YCoord(20);
   Height(20);
   Width(120);
   Text("");
   TextFont("Arial");
   TextColor(clrBlack);
   TextSize(10);
   ReadOnly(true);
   BorderColor(clrBlack);
   BackgroundColor(clrWhite);
   BorderType(BORDER_FLAT);
   Align(ALIGN_LEFT);
  }
//+------------------------------------------------------------------+
//|         :|
//|   - TypeElement;                                      |
//|    X = XCoord()                                     |
//|    X = YCoord()                                     |
//|  = Height()                                                |
//|  = Width()                                                 |
//|   = FontSize()                                       |
//+------------------------------------------------------------------+
void CElChart::Show(void)
  {
   ObjectCreate(ChartID(),m_name,m_obj_type,0,0,0);
   XCoord(XCoord());
   YCoord(YCoord());
   Height(Height());
   Width(Width());
   if(m_obj_type==OBJ_BITMAP_LABEL)
     {
      OnShow();
      return;
     }
   if(m_obj_type==OBJ_EDIT)
      Align(Align());
   Text(Text());
   TextColor(TextColor());
   TextSize(TextSize());
   TextFont(TextFont());
   ReadOnly(ReadOnly());
   Corner(Corner());
   BorderType(BorderType());
   if(BorderType()==BORDER_FLAT)
      BorderColor(BorderColor());
   BackgroundColor(BackgroundColor());

   OnShow();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CElChart::Event(CEvent *event)
  {
   CNode::Event(event);
   if(event.EventType()==EVENT_CHART_END_EDIT)
     {
      CEventChartEndEdit *endEdit=event;
      if(endEdit.ObjectName()==Name())
         m_text=ObjectGetString(ChartID(),Name(),OBJPROP_TEXT);
     }
  }
//+------------------------------------------------------------------+
//|      X                         |
//+------------------------------------------------------------------+
void CElChart::XCoord(long x)
  {
   m_x=x;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_XDISTANCE,m_x);
   OnXCoordChange();
  }
//+------------------------------------------------------------------+
//|      X                            |
//+------------------------------------------------------------------+
long CElChart::XCoord(void)
  {
   return m_x;
  }
//+------------------------------------------------------------------+
//|      Y                         |
//+------------------------------------------------------------------+
void CElChart::YCoord(long y)
  {
   m_y=y;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_YDISTANCE,m_y);
   OnYCoordChange();
  }
//+------------------------------------------------------------------+
//|      Y                            |
//+------------------------------------------------------------------+
long CElChart::YCoord(void)
  {
   return m_y;
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CElChart::Width(long width)
  {
   m_width=width;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_XSIZE,m_width);
   OnWidthChange();
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
long CElChart::Width(void)
  {
   return m_width;
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CElChart::Height(long height)
  {
   m_height=height;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_YSIZE,m_height);
   OnHeightChange();
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
long CElChart::Height(void)
  {
   return m_height;
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
void CElChart::Text(string text)
  {
   m_text=text;
   if(IsShowed())
      ObjectSetString(ChartID(),m_name,OBJPROP_TEXT,m_text);
  }
//+------------------------------------------------------------------+
//|                                           |
//+------------------------------------------------------------------+
string CElChart::Text(void)
  {
   return m_text;
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
void CElChart::TextSize(int text_size)
  {
   m_text_size=text_size;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_FONTSIZE,m_text_size);
  }
//+------------------------------------------------------------------+
//|                             |
//+------------------------------------------------------------------+
int CElChart::TextSize(void)
  {
   return m_text_size;
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
void CElChart::TextColor(color clr)
  {
   m_text_color=clr;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_COLOR,m_text_color);
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
color CElChart::TextColor(void)
  {
   return m_text_color;
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
void CElChart::TextFont(string text_font)
  {
   m_text_font=text_font;
   if(IsShowed())
      ObjectSetString(ChartID(),m_name,OBJPROP_FONT,m_text_font);
  }
//+------------------------------------------------------------------+
//|   .                                      |
//+------------------------------------------------------------------+
string CElChart::TextFont(void)
  {
   return m_text_font;
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
void CElChart::BackgroundColor(color clr)
  {
   m_bground_color=clr;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_BGCOLOR,m_bground_color);
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
color CElChart::BackgroundColor(void)
  {
   return m_bground_color;
  }
//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
void CElChart::BorderColor(color clr)
  {
   m_border_color=clr;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_BORDER_COLOR,m_border_color);
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
color CElChart::BorderColor(void)
  {
   return m_border_color;
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
void CElChart::BorderType(ENUM_BORDER_TYPE type)
  {
   m_border_type=type;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_BORDER_TYPE,m_border_type);
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
ENUM_BORDER_TYPE CElChart::BorderType(void)
  {
   return m_border_type;
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
void CElChart::ReadOnly(bool read_only)
  {
   if(m_obj_type==OBJ_EDIT)
     {
      m_read_only=read_only;
      if(IsShowed())
         ObjectSetInteger(ChartID(),m_name,OBJPROP_READONLY,m_read_only);
     }
  }
//+------------------------------------------------------------------+
//|                         |
//+------------------------------------------------------------------+
ENUM_BASE_CORNER CElChart::Corner(void)
  {
   return m_corner;
  }
//+------------------------------------------------------------------+
//|                      |
//+------------------------------------------------------------------+
void CElChart::Corner(ENUM_BASE_CORNER corner)
  {
   m_corner=corner;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_CORNER,corner);
   for(int i=0; i<m_elements.Total(); i++)
     {
      CElChart *el=m_elements.At(i);
      el.Corner(corner);
     }
  }
//+------------------------------------------------------------------+
//|     OBJ_EDIT                      |
//+------------------------------------------------------------------+
ENUM_ALIGN_MODE CElChart::Align(void)
  {
   return m_align;
  }
//+------------------------------------------------------------------+
//|    OBJ_EDIT                                 |
//+------------------------------------------------------------------+
void CElChart::Align(ENUM_ALIGN_MODE align)
  {
   m_align=align;
   if(IsShowed())
      ObjectSetInteger(ChartID(),m_name,OBJPROP_ALIGN,m_align);
  }
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
bool CElChart::ReadOnly(void)
  {
   return m_read_only;
  }
//+------------------------------------------------------------------+
//|          X|
//+------------------------------------------------------------------+
void CElChart::OnXCoordChange(void)
  {
  }
//+------------------------------------------------------------------+
//|          Y|
//+------------------------------------------------------------------+
void CElChart::OnYCoordChange(void)
  {
  }
//+------------------------------------------------------------------+
//|                  |
//+------------------------------------------------------------------+
void CElChart::OnHeightChange(void)
  {
  }
//+------------------------------------------------------------------+
//|                  |
//+------------------------------------------------------------------+
void CElChart::OnWidthChange(void)
  {
  }
//+------------------------------------------------------------------+
