//+------------------------------------------------------------------+
//|                                                 TradeControl.mqh |
//|           Copyright 2016, Vasiliy Sokolov, St-Petersburg, Russia |
//|                                https://www.mql5.com/ru/users/c-4 |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, Vasiliy Sokolov."
#property link      "https://www.mql5.com/ru/users/c-4"
#include <Object.mqh>
#include <Dictionary.mqh>
#include <Strategy\TradeCustom.mqh>
#include <Strategy\Message.mqh>
#include <Strategy\Logs.mqh>

/* 
             CTradeCustom. :
      1.        ,    
   ,   TradeDelayMsc,   ,
    ,           .
      2.          -
     ,    AddLimit,   
          .
*/

//+------------------------------------------------------------------+
//|  ,   CTradeCustom       |
//|    .                            |
//+------------------------------------------------------------------+
class CTradeControl : public CTradeCustom
  {
private:
   uint              m_last_trade_action;     //     .
   uint              m_trade_delay;           //    ,          .
   CLog*             Log;                     // 
public:
   CTradeCustom      Trade;                 //   .          .
                     CTradeControl(void);
/*   CTradeCustom */

   void              TradeDelayMsc(uint msec);
   void              AsynchMode(bool asynch);

/*    CTradeCustom */
   virtual bool      Buy(const double volume,const string symbol,const string comment="");
   virtual bool      Sell(const double volume,const string symbol,const string comment="");
   virtual bool      BuyLimit(const double volume,const double price,const string symbol,
                              const ENUM_ORDER_TYPE_TIME type_time=ORDER_TIME_GTC,const datetime expiration=0,const string comment="");
   virtual bool      BuyStop(const double volume,const double price,const string symbol,
                             const ENUM_ORDER_TYPE_TIME type_time=ORDER_TIME_GTC,const datetime expiration=0,const string comment="");
   virtual bool      SellLimit(const double volume,const double price,const string symbol,
                               const ENUM_ORDER_TYPE_TIME type_time=ORDER_TIME_GTC,const datetime expiration=0,const string comment="");
   virtual bool      SellStop(const double volume,const double price,const string symbol,
                              const ENUM_ORDER_TYPE_TIME type_time=ORDER_TIME_GTC,const datetime expiration=0,const string comment="");
  };
//+------------------------------------------------------------------+
//| .                                                     |
//+------------------------------------------------------------------+
CTradeControl::CTradeControl(void) : m_trade_delay(0),
                                     m_last_trade_action(0)
  {
   Log=CLog::GetLog();
  }
//+------------------------------------------------------------------+
//|       .      |
//+------------------------------------------------------------------+
void CTradeControl::AsynchMode(bool asynch)
  {
   Trade.SetAsyncMode(asynch);
  }
//+------------------------------------------------------------------+
//|   .                                               |
//+------------------------------------------------------------------+
bool CTradeControl::Buy(const double volume,const string symbol,const string comment="")
  {
   bool res=CTradeCustom::Buy(volume,symbol,comment);
   if(res)
      m_last_trade_action=GetTickCount();
   return res;
  }
//+------------------------------------------------------------------+
//|   .                                               |
//+------------------------------------------------------------------+
bool CTradeControl::Sell(const double volume,const string symbol,const string comment="")
  {
   bool res=CTradeCustom::Sell(volume,symbol,comment);
   if(res)
      m_last_trade_action=GetTickCount();
   return res;
  }
//+------------------------------------------------------------------+
//|  buy-stop.                                             |
//+------------------------------------------------------------------+
bool CTradeControl::BuyStop(const double volume,
                            const double price,
                            const string symbol,
                            const ENUM_ORDER_TYPE_TIME type_time=0,
                            const datetime expiration=0,
                            const string comment="")
  {
   bool res=CTradeCustom::BuyStop(volume,price,symbol,type_time,expiration,comment);
   if(res)
      m_last_trade_action=GetTickCount();
   return res;
  }
//+------------------------------------------------------------------+
//|  sell stop .                                      |
//+------------------------------------------------------------------+
bool CTradeControl::SellStop(const double volume,
                             const double price,
                             const string symbol,
                             const ENUM_ORDER_TYPE_TIME type_time=0,
                             const datetime expiration=0,
                             const string comment="")
  {
   bool res=CTradeCustom::SellStop(volume,price,symbol,type_time,expiration,comment);
   if(res)
      m_last_trade_action=GetTickCount();
   return res;
  }
//+------------------------------------------------------------------+
//|  buy limit .                                      |
//+------------------------------------------------------------------+
bool CTradeControl::BuyLimit(const double volume,
                             const double price,
                             const string symbol,
                             const ENUM_ORDER_TYPE_TIME type_time=0,
                             const datetime expiration=0,
                             const string comment="")
  {
   bool res=CTradeCustom::BuyLimit(volume,price,symbol,type_time,expiration,comment);
   if(res)
      m_last_trade_action=GetTickCount();
   return res;
  }
//+------------------------------------------------------------------+
//|  buy limit .                                      |
//+------------------------------------------------------------------+
bool CTradeControl::SellLimit(const double volume,
                              const double price,
                              const string symbol,
                              const ENUM_ORDER_TYPE_TIME type_time=0,
                              const datetime expiration=0,
                              const string comment="")
  {
   bool res=CTradeCustom::SellLimit(volume,price,symbol,type_time,expiration,comment);
   if(res)
      m_last_trade_action=GetTickCount();
   return res;
  }
//+------------------------------------------------------------------+
